# for pypdfium2-4.30.1
%global pdfium_build 6899

# https://github.com/pypdfium2-team/ctypesgen
%bcond_without ctypesgen
%global ctypesgen_commit ebd495b1733b60132151154d6358fd1eb336a36a

# https://pdfium.googlesource.com/pdfium/+/refs/heads/chromium/6899
%global pdfium_commit a7d54392b93dddf9c09c77a99c210a41914b3b8d
%global pdfium_commitdate 20241212
%global pdfium_shortcommit %(c=%{pdfium_commit}; echo ${c:0:8})

# git ls-remote --sort -version:refname --tags https://chromium.googlesource.com/chromium/src '*.*.6899.0'
# ...        refs/tags/133.0.6899.0
%global pdfium_major 133
%global pdfium_minor 0
%global pdfium_patch 0

# from https://pdfium.googlesource.com/pdfium/+/refs/heads/chromium/6899/DEPS
# local Chromium, PDFium wants specific commits of abseil-cpp and fast_float
# gtest for unit tests
%global build_revision 84fa09ff44bbdeef23b01d87078a63dac04dae8b
%global abseil_revision 0b76dfe4fffd3f49efacde5f3268b02f9c060ef0
%global fast_float_revision 3e57d8dcfb0a04b5a8a26b486b54490a2e9b310f
%global gtest_revision d144031940543e15423a25ae5a8a74141044862f


%if 0%{?fedora} || 0%{?rhel} >= 10
%bcond_without openjpeg2
%else
# RHEL 9.4 has OpenJPEG2 2.4, PDFium needs OpenJPEG2 >= 2.5
%bcond_with openjpeg2
%endif

Name:           libpdfium
Version:        %{pdfium_build}
Release:        1%{?dist}
Summary:        Library for PDF rendering, inspection, manipulation and creation

License:        Apache 2.0
URL:            https://pdfium.googlesource.com/pdfium
# Source0:        https://pdfium.googlesource.com/pdfium/+archive/refs/heads/chromium/%%{pdfium_build}.tar.gz#/pdfium-%%{pdfium_build}.tar.gz
Source0:        pdfium-%{pdfium_build}-clean.tar.gz
Source1:        args.gn
Source2:        passflags-BUILD.gn
Source3:        https://github.com/pypdfium2-team/ctypesgen/archive/%{ctypesgen_commit}.tar.gz#/ctypesgen-%{ctypesgen_commit}.tar.gz
Source5:        https://chromium.googlesource.com/chromium/src/build.git/+archive/%{build_revision}.tar.gz#/build-%{build_revision}.tar.gz
Source6:        https://chromium.googlesource.com/chromium/src/third_party/abseil-cpp/+archive/%{abseil_revision}.tar.gz#/abseil-cpp-%{abseil_revision}.tar.gz
Source7:        https://chromium.googlesource.com/chromium/src/+archive/refs/tags/%{pdfium_major}.%{pdfium_minor}.%{pdfium_build}.%{pdfium_patch}/tools/generate_shim_headers.tar.gz#/generate_shim_headers-%{pdfium_build}.tar.gz
Source8:        https://chromium.googlesource.com/external/github.com/fastfloat/fast_float.git/+archive/%{fast_float_revision}.tar.gz#/fast_float-%{fast_float_revision}.tar.gz
Source9:        https://chromium.googlesource.com/external/github.com/google/googletest.git/+archive/%{gtest_revision}.tar.gz#/gtest-%{gtest_revision}.tar.gz

BuildRequires:  gcc-c++
BuildRequires:  glibc-devel
BuildRequires:  pkgconfig
BuildRequires:  redhat-rpm-config
BuildRequires:  libatomic
BuildRequires:  chrpath

BuildRequires:  gn
BuildRequires:  ninja-build
%if %{with ctypesgen}
BuildRequires:  python3
%endif

# unbundled dependencies
BuildRequires:  pkgconfig(glib-2.0)
BuildRequires:  pkgconfig(freetype2)
BuildRequires:  pkgconfig(icu-uc)
BuildRequires:  pkgconfig(lcms2)
BuildRequires:  pkgconfig(libjpeg)
%if %{with openjpeg2}
BuildRequires:  pkgconfig(libopenjp2) >= 2.5
%endif
BuildRequires:  pkgconfig(libpng)
BuildRequires:  pkgconfig(libtiff-4)
BuildRequires:  pkgconfig(zlib)

# https://sourceforge.net/projects/agg/ 2.3 + security patches
Provides:       bundled(agg) = 2.3
%if %{without openjpeg2}
# OpenJPEG2 2.5.0 + security fixes
Provides:       bundled(openjpeg2) = 2.5.0
%endif
Provides:       bundled(abseil-cpp)
Provides:       bundled(fast_float)
Provides:       %{name}(build) = %{pdfium_build}
Provides:       %{name}(version) = %{pdfium_major}.%{pdfium_minor}.%{pdfium_build}.%{pdfium_patch}


%description
Library for PDF rendering, inspection, manipulation and creation

%package        devel
Summary:        Development files for %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}

%description    devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}.

%if %{with ctypesgen}
%package        ctypesgen
Summary:        ctypes bindings for %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}
Provides:       %{name}(ctypesgen) = %{ctypesgen_commit}

%description    ctypesgen
The %{name}-ctypesgen package contains ctypes bindings for %{name}. The
bindings were generated with https://github.com/pypdfium2-team/ctypesgen from
commit %{ctypesgen_commit}.
%endif


%prep
%setup -qTD -c -a 5 -n libpdfium-%{pdfium_build}/build
%setup -qTD -c -a 6 -n libpdfium-%{pdfium_build}/third_party/abseil-cpp
%setup -qTD -c -a 7 -n libpdfium-%{pdfium_build}/tools/generate_shim_headers
%setup -qTD -c -a 8 -n libpdfium-%{pdfium_build}/third_party/fast_float/src
%setup -qTD -c -a 9 -n libpdfium-%{pdfium_build}/third_party/googletest/src
%if %{with ctypesgen}
%setup -T -D -q -a 3 -n libpdfium-%{pdfium_build}
%endif
%setup -qTD -c -a 0 -n libpdfium-%{pdfium_build}

# unbundle ICU
mkdir -p third_party/icu
cp build/linux/unbundle/icu.gn third_party/icu/BUILD.gn

# Build static abseil-cpp
sed -i 's/component("absl")/static_library("absl")/' third_party/abseil-cpp/BUILD.gn

# empty gclient config
touch build/config/gclient_args.gni

# don't build test fonts
sed -i '/third_party\/test_fonts/d' testing/BUILD.gn

# build configuration
mkdir -p out/release
cp %{SOURCE1} out/release/args.gn
%if %{with openjpeg2}
echo "use_system_libopenjpeg2 = true" >> out/release/args.gn
%else
echo "use_system_libopenjpeg2 = false" >> out/release/args.gn
%endif

# custom flavor of GCC toolchain that passes CFLAGS, CXXFLAGS, etc.
mkdir -p build/toolchain/linux/passflags
cp %{SOURCE2} build/toolchain/linux/passflags/BUILD.gn

# generate Ninja files with build flags
%set_build_flags
# build system does not define macro for some dependencies
export CPPFLAGS="-DUSE_SYSTEM_LCMS2=1 $CPPFLAGS"
gn gen out/release


%build
# build libpdfium.so
%ninja_build -C out/release pdfium pdfium_unittests

# remove rpath
chrpath --delete out/release/libpdfium.so

# generate libpdfium pkgconfig file
cat > libpdfium.pc <<EOF
prefix=%{_prefix}
exec_prefix=%{_exec_prefix}
libdir=%{_libdir}
includedir=%{_includedir}

Name: %{name}
Description: Library for PDF rendering, inspection, manipulation and creation
Version: %{version}
Cflags: -I\${includedir}
Requires.private:
Libs: -L\${libdir} -lpdfium
EOF

%if %{with ctypesgen}
# pypdfium2 ctypesgen bindings
# https://github.com/pypdfium2-team/pypdfium2/blob/4.30.0/autorelease/bindings.py
# https://github.com/pypdfium2-team/pypdfium2/blob/4.30.0/setupsrc/pypdfium2_setup/packaging_base.py

mkdir -p out/ctypesgen

PYTHONPATH=ctypesgen-%{ctypesgen_commit}/src python3 -m ctypesgen \
    -l pdfium \
    --no-load-library \
    --no-macro-guards \
    -D PDF_ENABLE_V8 PDF_ENABLE_XFA PDF_USE_SKIA \
    --symbol-rules 'if_needed=\w+_$|\w+_t$|_\w+' \
    --headers ./public/fpdf*.h \
    -o out/ctypesgen/bindings.py

# remove 'public/' prefix from comments
sed -i 's,public/,,g' out/ctypesgen/bindings.py

cat > "out/ctypesgen/version.json" << EOF
{
  "major": %{pdfium_major},
  "minor": %{pdfium_minor},
  "build": %{pdfium_build},
  "patch": %{pdfium_patch},
  "n_commits": 0,
  "hash": "%{pdfium_commit}",
  "origin": "system",
  "flags": []
}
EOF

%endif


%check
out/release/pdfium_unittests


%install
mkdir -p %{buildroot}%{_libdir}
cp out/release/libpdfium.so %{buildroot}%{_libdir}

mkdir -p %{buildroot}%{_includedir}/pdfium/cpp
cp public/*.h %{buildroot}%{_includedir}/pdfium
cp public/cpp/*.h %{buildroot}%{_includedir}/pdfium/cpp

mkdir -p %{buildroot}%{_libdir}/pkgconfig
cp libpdfium.pc %{buildroot}%{_libdir}/pkgconfig

%if %{with ctypesgen}
mkdir -p %{buildroot}%_datarootdir/%{name}/ctypesgen/
cp out/ctypesgen/* %{buildroot}%_datarootdir/%{name}/ctypesgen/
%endif

%files
%license LICENSE
%doc AUTHORS out/release/args.gn
# pypdfium2 needs "libpdfium.so"
%{_libdir}/*.so

%files devel
%doc README.md
%{_includedir}/pdfium
%{_libdir}/pkgconfig/libpdfium.pc

%if %{with ctypesgen}
%files ctypesgen
%_datarootdir/%{name}/ctypesgen/
%endif

%changelog
* Mon Mar 10 2025 Christian Heimes <cheimes@redhat.com> - 6899-1
- Update to PDFium 6899
- Rework build system

* Wed Oct 23 2024 Christian Heimes <cheimes@redhat.com> - 6462^20240502git7b7c83fb-4
- Rebuild SRPM without bug_867501.pdf. The test file is flagged as malicious
  content.

* Tue Oct 22 2024 Christian Heimes <cheimes@redhat.com> - 6462^20240502git7b7c83fb-3
- Build ctypes bindings with ctypesgen

* Fri Oct 11 2024 Christian Heimes <cheimes@redhat.com> - 6462^20240502git7b7c83fb-2
- Rebuild for RHELAI 1.3

* Fri Sep 27 2024 Christian Heimes <cheimes@redhat.com> - 6462^20240502git7b7c83fb-1
- Build tag 6462 for pypdfium2-4.30.0
- Resolves RHELAI-1562
